library(ade4) 
data(butterfly)
X<-butterfly$env 

#### Pairs plot

panel.cor <- function(x, y, digits = 2, prefix = "", cex.cor, ...)
{
    usr <- par("usr"); on.exit(par(usr))
    par(usr = c(0, 1, 0, 1))
    r <- cor(x, y)
    txt <- format(c(r, 0.123456789), digits = digits)[1]
    txt <- paste0(prefix, txt)
    if(missing(cex.cor)) cex.cor <- 0.8/strwidth(txt)
    text(0.5, 0.5, txt, cex = cex.cor * abs(r))
}
panel.hist <- function(x, ...)
{
    usr <- par("usr"); on.exit(par(usr))
    par(usr = c(usr[1:2], 0, 1.5) )
    h <- hist(x, plot = FALSE)
    breaks <- h$breaks; nB <- length(breaks)
    y <- h$counts; y <- y/max(y)
    rect(breaks[-nB], 0, breaks[-1], y, col = "lightgray", ...)
}

pairs(X, lower.panel = panel.cor,diag.panel=panel.hist)

## Correlations in Table 1.1

rob1<-CovMcd(X,alpha=0.75)
rob2<-CovMMest(X)
as.dist(round(cor(X),2))
as.dist(round(getCorr(rob1),2))
as.dist(round(getCorr(rob2),2))


### classical PCA on the correlation matrix
pc<-princomp(X,cor=TRUE)


### robust PCA based on MCD correlations via eigen analysis
robpca<-eigen(getCorr(rob1))
### robust standardized data matrix
Xc<-sapply(1:4,function(j) (X[,j]-rob1@center[j])/sqrt(rob1@cov[j,j]))
### scores
robscores<-Xc%*%robpca$vector

## Figure 1.4

plot(pc$scores, xlim=c(-6,6),ylim=c(-2,2))
abline(h=0,v=0)
p=ncol(X)
n=nrow(X)
qb1<-((n-1)^2)*qbeta(.90,p/2,(n-p-1)/2)/n
lines(ellipse(x=diag(pc$sdev[1:2]^2),center=rep(0,2),level=0.95, t= sqrt(qb1)),lty=1)
identify(pc$scores, label=rownames(X))
plot(robscores[,1:2],ylim=c(-3,5),xlim=c(-8,4),xlab='Comp.1',ylab='Comp.2')
abline(h=0,v=0)
n=sum(rob1@wt)
qb2<-((n-1)^2)*qbeta(.95,p/2,(n-p-1)/2)/n
lines(ellipse(x=diag(robpca$val[1:2]),center=rep(0,2),level=0.95, t= sqrt(qb2)),lty=1)
identify(robscores,  label=rownames(X))
