### Birth and death rates in Italy

library(foreign)
demoita=read.dbf('demographic.dbf')

## process data
asc<-demoita$Birthrate[-c(5,6,23)]
ord<-demoita$Deathrate[-c(5,6,23)]
X<-cbind(asc,ord)

## MLE
mle<-list(m=colMeans(X),S=cov(X))


#### RMCD BP 25% and MM 25% BP 95% shape eff

library(rrcov)
source(CovMcdF.r)

mcd<-CovMcdF(X, alpha=0.75)
mm<-CovMMest(X, bdp=0.25, eff=0.95, eff.shape=TRUE)

## Figure 2.11 

plot(asc,ord,xlim=c(3,14),ylim=c(5,16),xlab="Birthrate",ylab="Deathrate")

###maximum likelihood
cc=((nrow(X)-1)^2)/nrow(X)
quantmle<-qbeta(.975,ncol(X)/2, (nrow(X)-ncol(X)-1)/2)*cc
lines(ellipse(x=cov(X),centre=colMeans(X), level=0.975, t=sqrt(quantmle)),lty=3)


##RMCD (tolerance ellipse for the bulk of the data)

nw<-sum(mcd$wt)
ccw1=((nw-1)^2)/nw
quantrmcd<-qbeta(.975,ncol(X)/2, (nw-ncol(X)-1)/2)*ccw1
p=2
ccw2=(nw^2-1)*p/nw/(nw-p)
quantout<-qf(.975,p, nw-p)*ccw2
lines(ellipse(x=mcd$cov,centre=mcd$center, level=0.975, t=sqrt(quantrmcd)),lty=1)

## MM 
lines(ellipse(x=mm@cov,centre=mm@center, level=0.975),lty=2)

## graphical detection
identify(asc,ord,labels=demoita[-c(5,6,23),1])

##legend

legend('bottomleft', c('MLE','RMCD','MM'),lty=c(3,1,2))

## Figure 2.12

##### Distance plot

plot(sqrt(mcd$mah),type='n',ylab='Robust Distance', xlab='')
abline(h=sqrt(quantout))
abline(h=sqrt(quantrmcd),lty=2)
inx.bad<-which(mcd$wt==0)
inx.good<-which(mcd$wt!=0)
points(inx.good,mcd$mah[inx.good]^0.5,pch=1)
points(inx.bad,mcd$mah[inx.bad]^0.5,pch=2)
identify(sqrt(mcd$mah),labels=demoita[-c(5,6,23),1])

### Distance-distance plot

mle.mah<-mahalanobis(X,mle$m,mle$S)
plot(sqrt(mle.mah), sqrt(mcd$mah), xlab='Mahalanobis distance', ylab='Robust distance',type='n')
points(sqrt(mle.mah[inx.good]),sqrt(mcd$mah[inx.good]),pch=1)
points(sqrt(mle.mah[inx.bad]),sqrt(mcd$mah[inx.bad]),pch=2)
abline(v=sqrt(quantmle), h=sqrt(quantout))
abline(h=sqrt(quantrmcd),lty=2)
identify(sqrt(mle.mah), sqrt(mcd3b$mah), labels=demo[-c(5,6,23),1])

## Figure 2.13

plot(sqrt(mle.mah), sqrt(mcd$mah), xlab='Mahalanobis distance', ylab='Robust distance')
abline(v=sqrt(qchisq(.975,2)))
abline(h=sqrt(qchisq(.975,2)))

mm.mah<-mahalanobis(X,mm@center,mmb@cov)
plot(sqrt(mle.mah), sqrt(mm.mah), xlab='Mahalanobis distance', ylab='Robust distance')
abline(v=sqrt(qchisq(.975,2)))
abline(h=sqrt(qchisq(.975,2)))

