##### Sensitivity analyses from Examples 2.6 and 2.7

library(rrcov)
y<-read.table('ybivariate.txt')

### Maximum likelihood
plot(y,xlim=c(-6,6),xlab='outlier',ylab='')
dd0<-det(cov(y))
ee0<-eigen(cov(y))
er0<-ee0$val[1]/ee0$val[2]
N0<-norm(as.matrix(colMeans(y)), type='F')
outlier<-seq(-10,10,l=100)
dd<-er<-N<-NULL
for(i in 1:100)
{
	yn<-c(outlier[i],0)
	yn<-rbind(y,yn)
	m<-colMeans(yn)
	N[i]<-norm(as.matrix(m), type='F')
	S<-cov(yn)
	dd[i]<-det(S)
	ee<-eigen(S)
	er[i]<-ee$val[1]/ee$val[2]
	
}

plot(outlier, N-N0,type='l',xlab='outlier',ylab='Mean Norm')
plot(outlier, (dd-dd0),type='l',xlab='Outlier',ylab='Determinat')
plot(outlier, (er-er0),type='l',xlab='Outlier',ylab='Eigenvalues ratio')

#### Robust methods

out1<- CovMMest(y)
out2<-CovMcd(y)
	dd0<-er0<-N0<-NULL
	dd0[1]<-det(out1@sest@cov)
	dd0[2]<-det(out1@cov)
	dd0[3]<-det(out2@cov)
	ee1<-eigen(out1@sest@cov)
	er0[1]<-ee1$val[1]/ee1$val[2]
   ee2<-eigen(out1@cov)
	er0[2]<-ee2$val[1]/ee2$val[2]
	ee3<-eigen(out2@cov)
	er0[3]<-ee3$val[1]/ee3$val[2]
N0[1]<-norm(as.matrix(out1@sest@center), type='F')
N0[2]<-norm(as.matrix(out1@center), type='F')
N0[3]<-norm(as.matrix(out2@center), type='F')
outlier<-seq(-10,10,l=100)
dd<-er<-N<-matrix(0,nrow=100,ncol=3)


for(i in 1:100)
{
	yn<-c(outlier[i],0)
	yn<-rbind(y,yn)
	out1<- CovMMest(yn)
	out2<-CovMcd(yn)
	dd[i,1]<-det(out1@sest@cov)
	dd[i,2]<-det(out1@cov)
	dd[i,3]<-det(out2@cov)
	ee1<-eigen(out1@sest@cov)
	er[i,1]<-ee1$val[1]/ee1$val[2]
   ee2<-eigen(out1@cov)
	er[i,2]<-ee2$val[1]/ee2$val[2]
	ee3<-eigen(out2@cov)
	er[i,3]<-ee3$val[1]/ee3$val[2]
	N[i,1]<-norm(as.matrix(out1@sest@center), type='F')
	N[i,2]<-norm(as.matrix(out1@center), type='F')
	N[i,3]<-norm(as.matrix(out2@center), type='F')
}



plot(outlier, (N[,3]-N0[3]),type='l',xlab='Outlier',ylab='Mean norm',ylim=c(-0.2,0.2))
lines(outlier, (N[,2]-N0[2]),lty=2)
lines(outlier, (N[,1]-N0[1]),lty=3)
legend('topleft',c('MCD','MM','S'),lty=1:3)

plot(outlier, (dd[,3]-dd0[3]),type='l',xlab='Outlier',ylab='Determinat',ylim=c(-0.4,0.4))
lines(outlier, (dd[,2]-dd0[2]),lty=2)
legend('topleft',c('MCD','MM and S'),lty=1:2)

plot(outlier, (er[,3]-er0[3]),type='l',xlab='Outlier',ylab='Eigenvalues ratio',ylim=c(-1.5,1.5))
lines(outlier, (er[,2]-er0[2]),lty=2)
lines(outlier, (er[,1]-er0[1]),lty=3)
legend('topleft',c('MCD','MM','S'),lty=1:3)

