
library(rrcov)
library(ellipse)
source(''CovMcdF.r'')

## Estimation and Figure 2.14

data(starsCYG)
plot(starsCYG,xlim=c(3,5.5),ylim=c(3,7), xlab='log temperature', ylab='log intensity')
identify(starsCYG)


#MLE
mle.stars<-list(estcov=cov(starsCYG),estcent=colMeans(starsCYG))
ps<-ncol(starsCYG)
ns<-nrow(starsCYG)
cc=((ns-1)^2)/ns
quantmle<-qbeta(.975,ps/2, (ns-ps-1)/2)*cc
lines(ellipse(centre=mle.stars$estcent,x=mle.stars$estcov,t=sqrt(quantmle),level=0.975),lty=3)

### RMCD
rmcd.stars<-CovMcdF(starsCYG,alpha=0.5)
nw<-sum(rmcd.stars$wt)
ccw1=((nw-1)^2)/nw
quantrmcd<-qbeta(.975,ps/2, (nw-ps-1)/2)*ccw1
ccw2=(nw^2-1)*ps/nw/(nw-ps)
quantout<-qf(.975,ps, nw-ps)*ccw2
lines(ellipse(centre=rmcd.stars$center,x=rmcd.stars$cov,t=sqrt(quantrmcd),level=0.975),lty=1)

## MM
mm.stars<-CovMMest(starsCYG,bdp=0.5)
lines(ellipse(centre=mm.stars@center,x=mm.stars@cov,level=0.975),lty=2)
lines(ellipse(centre=mm.stars@sest@center,x=mm.stars@sest@cov,level=0.975),lty=4)
legend('bottomleft', c('MLE','RMCD','MM','S'), lty=c(3,1,2,4))

### Distance-distance plot


m.stars<-mahalanobis(starsCYG,mle.stars$estcent,mle.stars$estcov)
plot(sqrt(m.stars), sqrt(rmcd.stars$mah), xlab='Mahalanobis distance', ylab='Robust distance',type='n', main='RMCD')
inx.bad<-which(rmcd.stars$wt==0)
inx.good<-which(rmcd.stars$wt!=0)
points(sqrt(m.stars[inx.good]),sqrt(rmcd.stars$mah[inx.good]),pch=1)
points(sqrt(m.stars[inx.bad]),sqrt(rmcd.stars$mah[inx.bad]),pch=2)

abline(v=sqrt(quantmle))
abline(h=sqrt(c(quantrmcd,quantout)),lty=c(2,1))
identify(sqrt(m.stars), sqrt(rmcd.stars$mah))

mm.stars@mah<-mahalanobis(starsCYG,mm.stars@center,mm.stars@cov)
plot(sqrt(m.stars), sqrt(mm.stars@mah), xlab='Mahalanobis distance', ylab='Robust distance', main='MM')
abline(v=sqrt(qchisq(.975,2)),h=sqrt(qchisq(.975,2)))
identify(sqrt(m.stars), sqrt(mm.stars@mah))

### Monitoring the BP of S estimation

bp<-seq(0.5,0.01,by=0.01)
mat.dist<-matrix(0,ncol=length(bp),nrow=nrow(starsCYG))
for(i in 1:length(bp))
{
mm.stars.out<-CovSest(starsCYG,bdp=bp[i])
mat.dist[,i]<-sqrt(mahalanobis(starsCYG,mm.stars.out@center,mm.stars.out@cov))	
}

### Monitoring efficiency of MM estimation

eff<-seq(0.7,0.99,by=0.01)
mat.dist<-matrix(0,ncol=length(eff),nrow=nrow(starsCYG))

for(i in 1:length(eff))
{
mm.stars.out<-CovMMest(starsCYG,eff=eff[i],eff.shape=FALSE,bdp=0.35)
mat.dist[,i]<-sqrt(mahalanobis(starsCYG,mm.stars.out@center,mm.stars.out@cov))	
}

### Monitoring the BP of RMCD

bp<-seq(0.5,0.01,by=-0.01)
mat.dist<-matrix(0,ncol=length(bp),nrow=nrow(starsCYG))
for(i in 1:length(bp))
{
mcd.stars.out<-CovMcdF(starsCYG,alpha=1-bp[i])
mat.dist[,i]<-sqrt(mcd.stars.out$mah)	
}
