library(robustbase)
source('univariate_fun.r')
## Example 2.4 Simulated univariate Gaussian

y<-c(28.69,  23.81,  27.27, 26.49,  26.95,  22.24,
29.86, 24.59, 27.25, 
26.16, 22.76, 25.60, 23.75, 16.54, 23.58, 24.66, 25.43, 24.14,
22.74, 31.08, 26.06, 22.66, 29.91, 15.34, 28.85, 23.88, 24.03, 
31.93, 22.15, 27.95)

## contaminated sample
yc<-y; yc[which.max(yc)]<-61.93

### Asymptotic confidence interval based on median and MAD
low.mm.z<-median(yc)-qnorm(.975)*mad(yc)*sqrt(pi/2)/sqrt(30)
upp.mm.z<-median(yc)+qnorm(.975)*mad(yc)*sqrt(pi/2)/sqrt(30)
round(c(low.mm.z,upp.mm.z),2)


#### Asymptotic confidence interval based on Huber with MAD 

h1<-huberM(yc,k=1.345, s=mad(yc),se=T)
## this interval is obtained by using the empirical correction factor
low.HM1<-h1$mu-qnorm(.975)*h1$SE; upp.HM1<-h1$mu+qnorm(.975)*h1$SE
round(c(h1$mu,low.HM1,upp.HM1),2)

## this interval is obtained by using the asymptotic correction factor
low.HM1b<-h1$mu-qnorm(.975)*h1$s/sqrt(areH(1.345))/sqrt(n); upp.HM1b<-h1$mu+qnorm(.975)*h1$s/sqrt(areH(1.345))/sqrt(n)
round(c(h1$mu,low.HM1b,upp.HM1b),2)

#### Asymptotic confidence interval based on Huber with Qn

h2<-huberM(yc,k=1.345, s=Qn(y),se=T)
## this interval is obtained by using the empirical correction factor
low.HM2<-h2$mu-qnorm(.975)*h2$SE; upp.HM2<-h2$mu+qnorm(.975)*h2$SE
round(c(h2$mu,low.HM2,upp.HM2),2)

## this interval is obtained by using the asymptotic correction factor
c95<-T.find.c(0.95)
low.HM2b<-h2$mu-qnorm(.975)*h2$s/sqrt(areT(c95))/sqrt(n); upp.HM2b<-h2$mu+qnorm(.975)*h2$s/sqrt(areT(c95))/sqrt(n)
round(c(h2$mu,low.HM2b,upp.HM2b),2)

#### Asymptotic confidence interval based on Tukey and function lmrob

T1<-summary(lmrob(y~1,control=lmrob.control(tuing.psi=c95)))
low<-T1$coef[1]-qnorm(.975)*T1$coef[2]; upp<-T1$coef[1]+qnorm(.975)*T1$coef[2]
round(c(T1$coef[1],low,upp),2)

## this interval is obtained by using the asymptotic correction factor
lowb<-T1$coef[1]-qnorm(.975)*T1$scale/sqrt(n)/areT(c95); uppb<-T1$coef[1]+qnorm(.975)*T1$scale/sqrt(n)/areT(c95)
round(c(T1$coef[1],lowb,uppb),2)





