library(rrcov);library(ellipse)
library(pcaPP)



####### Octane data


X<-read.csv('Octane.csv',header=T,dec=',')
X<-X[,-(1:2)]

## ROBPCA with 2 components
rob<-PcaHubert(X, k=2,alpha=0.75,scale=FALSE)

## PP

rob.pp<-PCAgrid(X,k=2,method='mad')
rob.pp2<-PCAgrid(X,k=2,method='qn')
summary(rob.pp)

##### Spherical PCA
rob.loc<-PcaLocantore(X,k=2)
summary(rob.loc)

## Figure 3.10
pc.octane<-eigen(cov(X))
plot(pc.octane$vector[,1],type='l', ylab='Loadings', xlab='Wavenumber',main='PC1')
plot(pc.octane$vector[,2],type='l', ylab='Loadings', xlab='Wavenumber',main='PC2')


#### Figure 3.11 
plot(-rob.pp$loadings[,1],type='l',col='1',ylim=c(-0.3,0.2),
ylab='Loadings', xlab='Wavenumber', lty=2,main='PC1')
lines(-rob.pp2$loadings[,1],lty=3)
lines(rob@loadings[,1])
lines(-rob.loc@loadings[,1],lty=4)
legend('bottomright', lty=1:4, c('ROBPCA', 'MAD-PP', 'Qn-PP','SPCA'))

plot(-rob.pp$loadings[,2],type='l',col='1',ylim=c(-0.2,0.2),
ylab='Loadings', xlab='Wavenumber', lty=2,main='PC2')
lines(rob.pp2$loadings[,2],lty=3)
lines(rob@loadings[,2])
lines(rob.loc@loadings[,2],lty=4)
legend('bottomright', lty=1:4, c('ROBPCA', 'MAD-PP', 'Qn-PP','SPCA'))

#### Figure 3.12

out<-c(25:26,36:39)
Xc<-sapply(1:ncol(X), function(j) X[,j]-colMeans(X)[j])
Y.pca<- Xc%*%pc.octane$vector[,1:2]

plot(ellipse(diag(pc.octane$values[1:2])),ylab='PC2', xlab='PC1',main='PCA',type='l',xlim=c(-1.5,1.5))
points(Y.pca[-out,1],Y.pca[-out,2])
points(Y.pca[out,1],Y.pca[out,2],pch='+')
abline(h=0,v=0)


plot(rob@scores[-out,1],rob@scores[-out,2], ylab='PC2', xlab='PC1',main='ROBPCA',
xlim=c(-0.4,0.4),ylim=c(-0.4,0.2))
points(rob@scores[out,1],rob@scores[out,2],pch='+')
lines(ellipse(diag(rob@eigenvalues[1:2])))
abline(h=0,v=0)


plot(rob.pp$scores[-out,1],rob.pp$scores[-out,2], ylab='PC2', xlab='PC1',main='PP',xlim=c(-0.5,0.5),ylim=c(-0.2,0.8))
points(rob.pp$scores[out,1],rob.pp$scores[out,2],pch='+')
lines(ellipse(diag(rob.pp$sdev[1:2]^2)))
abline(h=0,v=0)


plot(rob.loc@scores[-out,1],rob.loc@scores[-out,2], ylab='PC2', xlab='PC1',main='SPCA',xlim=c(-0.5,0.6),ylim=c(-1.3,0.2))
points(rob.loc@scores[out,1],rob.loc@scores[out,2],pch='+')
lines(ellipse(diag(rob.loc@eigenvalues[1:2])))
abline(h=0,v=0)


#### Figure 3.13

## ROBPCA
plot(rob)
identify(m1@sd,m1@od)

#### PP
out<-PCdiagplot(X,rob.pp,plot=F,raw=T,ksel=2,crit=0.975 )
plot(out$SDist[,1],out$ODist[,1], ylab='OD', xlab='SD',main='PP')
abline(h=out$critOD,v=out$critSD)
identify(out$SDist[,1],out$ODist[,1])


### SPCA
plot(rob.loc)
identify(rob.loc@sd,rob.loc@od)


## PCA
X0<-sapply(1:ncol(X), function(j) X[,j]-colMeans(X)[j])
pc.octane$scores<-X0%*%pc.octane$vector[,1:2]
sd<-sqrt(mahalanobis(pc.octane$scores, rep(0,2), diag(pc.octane$values[1:2])))
tildeX<-pc.octane$scores[,1:2]%*%t(pc.octane$vector[,1:2])
resX<-X0-tildeX
od<-sapply(1:nrow(X), function(i) sqrt(sum(resX[i,]^2)))
odn<-od^(2/3)
c.od<-(median(odn)+qnorm(.975)*mad(odn))^(3/2)

plot(sd,od, ylab='OD', xlab='SD',main='PCA')
abline(h=c.od,v=rob@cutoff.sd)
identify(sd,od)

### ROBPCA reconstruction

Xrc<-sapply(1:ncol(X), function(j) X[,j]-rob@center[j])
Xtilde<-rob@scores%*%t(rob@loadings)
Xtilde<-sapply(1:ncol(X), function(j) Xtilde[,j]+m1@center[j])
plot(Xtilde[1,], type='l',  xlab='Wavenumber', ylab='', ylim=c(0,0.7))
for(i in 2:39) lines(Xtilde[i,])

## PCA reconstruction

Xc<-sapply(1:ncol(X), function(j) X[,j]-colMeans(X)[j])
Z.pca<-Xc%*%pc.octane$vector[,1:2]
Xtilde1<-Z.pca%*%t(pc.octane$vector[,1:2])
Xtilde1<-sapply(1:ncol(X), function(j) Xtilde1[,j]+colMeans(X)[j])
plot(Xtilde1[1,], type='l',  xlab='Wavenumber', ylab='', , ylim=c(0,0.7))
for(i in 2:39) lines(Xtilde1[i,])
