library(rrcov);library(ellipse)
source('CovMcdF.r')

#### Automobile data
auto<-read.table('auto.dat',sep='',header=TRUE)

## Standard PCA

outcla<-princomp(autonew[,-2], cor=TRUE)
summary(outcla)

### Score-plots
plot(outcla$scores[autonew[,2]=='gas',1],outcla$scores[autonew[,2]=='gas',2],
xlim=range(outcla$scores[,1])+c(0,0.3),ylim=range(outcla$scores[,2])+c(0,0.3),
ylab='PC2', xlab='PC1')
points(outcla$scores[autonew[,2]=='diesel',1],outcla$scores[autonew[,2]=='diesel',2],pch='+')
abline(h=0,v=0)
lines(ellipse(x=diag(outcla$sdev[1:2]^2),level=0.975))


plot(outcla$scores[autonew[,2]=='gas',1],outcla$scores[autonew[,2]=='gas',3],
xlim=range(outcla$scores[,1])+c(0,0.3),ylim=range(outcla$scores[,3])+c(-1,1),
ylab='PC3', xlab='PC1')
points(outcla$scores[autonew[,2]=='diesel',1],outcla$scores[autonew[,2]=='diesel',3],pch='+')
abline(h=0,v=0)
lines(ellipse(x=diag(outcla$sdev[c(1,3)]^2),level=0.975))

plot(outcla$scores[autonew[,2]=='gas',2],outcla$scores[autonew[,2]=='gas',3],
xlim=range(outcla$scores[,2])+c(0,0.3),ylim=range(outcla$scores[,3])+c(-1,1),
ylab='PC3', xlab='PC2')
points(outcla$scores[autonew[,2]=='diesel',1],outcla$scores[autonew[,2]=='diesel',3],pch='+')
abline(h=0,v=0)
lines(ellipse(x=diag(outcla$sdev[c(2,3)]^2),level=0.975))

### Robust PCA based on RMCD and MM
### Data have been scaled by columnwise MAD
x.mad <- apply(auto[,-2], 2, mad)
xmad <- sweep(auto[,-2], 2, x.mad, "/", check.margin = FALSE)
outrobmcd<-PcaCov(xmad, scale=FALSE, cov.control=CovControlMcd(alpha=0.75))
outrobmm<-PcaCov(xmad, scale=FALSE, cov.control=CovControlMMest())

### Scree-plots
par(mfrow=c(1,3))
screeplot(outcla, type='l',main='Classic')
screeplot(outrobmcd, type='l',main='RMCD')
screeplot(outrobmm, type='l',main='MM')


### Score-plots

plot(outrobmcd@scores[autonew[,2]=='gas',1],outrobmcd@scores[autonew[,2]=='gas',2],
xlim=range(outrobmcd@scores[,1])+c(0,2),ylim=range(outrobmcd@scores[,2])+c(0,4),
ylab='PC2', xlab='PC1')
points(outrobmcd@scores[autonew[,2]=='diesel',1],outrobmcd@scores[autonew[,2]=='diesel',2],pch='+')
abline(v=0,h=0)
lines(ellipse(x=diag(outrobmcd@eigenvalues[1:2]),level=0.975))

plot(outrobmcd@scores[autonew[,2]=='gas',1],outrobmcd@scores[autonew[,2]=='gas',3],
xlim=range(outrobmcd@scores[,1]),ylim=range(outrobmcd@scores[,3])+c(-1,2),
ylab='PC3', xlab='PC1')
points(outrobmcd@scores[autonew[,2]=='diesel',1],outrobmcd@scores[autonew[,2]=='diesel',3],pch='+')
abline(v=0,h=0)
lines(ellipse(x=diag(outrobmcd@eigenvalues[c(1,3)]),level=0.975))


plot(outrobmcd@scores[autonew[,2]=='gas',2],outrobmcd@scores[autonew[,2]=='gas',3],
xlim=range(outrobmcd@scores[,2])+c(0,2),ylim=range(outrobmcd@scores[,3])+c(-1,2),
ylab='PC3', xlab='PC2')
points(outrobmcd@scores[autonew[,2]=='diesel',2],outrobmcd@scores[autonew[,2]=='diesel',3],pch='+')
abline(v=0,h=0)
lines(ellipse(x=diag(outrobmcd@eigenvalues[c(2,3)]),level=0.975))


plot(outrobmm@scores[autonew[,2]=='gas',1],outrobmm@scores[autonew[,2]=='gas',2],
xlim=range(outrobmm@scores[,1])+c(0,2),ylim=range(outrobmm@scores[,2])+c(0,4),
ylab='PC2', xlab='PC1')
points(outrobmm@scores[autonew[,2]=='diesel',1],outrobmm@scores[autonew[,2]=='diesel',2],pch='+')
abline(v=0,h=0)
lines(ellipse(x=diag(outrobmm@eigenvalues[1:2]),level=0.975))

plot(outrob@scores[autonew[,2]=='gas',1],outrob@scores[autonew[,2]=='gas',3],
xlim=range(outrob@scores[,1]),ylim=range(outrob@scores[,3])+c(-1,2),
ylab='PC3', xlab='PC1')
points(outrob@scores[autonew[,2]=='diesel',1],outrob@scores[autonew[,2]=='diesel',3],pch='+')
abline(v=0,h=0)
lines(ellipse(x=diag(outrob@eigenvalues[c(1,3)]),level=0.975))


plot(outrob@scores[autonew[,2]=='gas',2],outrob@scores[autonew[,2]=='gas',3],
xlim=range(outrob@scores[,2])+c(0,2),ylim=range(outrob@scores[,3])+c(-1,2),
ylab='PC3', xlab='PC2')
points(outrob@scores[autonew[,2]=='diesel',2],outrob@scores[autonew[,2]=='diesel',3],pch='+')
abline(v=0,h=0)
lines(ellipse(x=diag(outrob@eigenvalues[c(2,3)]),level=0.975))

#### Distance-distance plot based on 3 PCs

k=3

#### Classical

SD.cla<-sqrt(mahalanobis(outcla$scores[,1:k],rep(0,k), diag(outcla$sd[1:k]^2)))
tildeX.cla<-outcla$scores[,1:k]%*%t(outcla$loadings[,1:k]) 
Xcla<-scale(auto[,-2])
resX<-Xcla-tildeX.cla
OD.cla<-sapply(1:nrow(autonew), function(j) sqrt(sum(resX[j,]^2)))
OD.cla.tr<-OD.cla^(2/3)
cutoff.OD.cla=(mean(OD.cla.tr)+qnorm(.975)*sd(OD.cla.tr))^(3/2)

plot(SD.cla[autonew[,2]=='gas'],OD.cla[autonew[,2]=='gas'],
xlim=range(SD.cla),ylim=range(OD.cla),xlab='SD',ylab='OD')
points(SD.cla[autonew[,2]=='diesel'],OD.cla[autonew[,2]=='diesel'], pch='+')
abline(v=sqrt(qchisq(3.975,k)), h=cutoff.OD.cla)
identify(SD.cla,OD.cla)

### MCD

SD.mcd<-sqrt(mahalanobis(outrob@scores[,1:k],rep(0,k), diag(outrob@eigenvalues[1:k])))
tildeX.mcd<-outrob@scores[,1:k]%*%t(outrob@loadings[,1:k]) 
Xmcdc<-sapply(1:ncol(autonew[,-2]),function(j) xmad[,j]-outrob@center[j])
resX<-Xmcdc-tildeX.mcd
OD.mcd<-sapply(1:nrow(autonew), function(j) sqrt(sum(resX[j,]^2)))
OD.mcd.tr<-OD.mcd^(2/3)
cutoff.OD.mcd=(median(OD.mcd.tr)+qnorm(.975)*mad(OD.mcd.tr))^(3/2)

plot(SD.mcd[autonew[,2]=='gas'],OD.mcd[autonew[,2]=='gas'],
 xlim=range(SD.mcd),ylim=range(OD.mcd),xlab='SD',ylab='OD')
points(SD.mcd[autonew[,2]=='diesel'],OD.mcd[autonew[,2]=='diesel'], pch='+')
abline(v=sqrt(qchisq(.975,k)), h=cutoff.OD.mcd)
identify(SD.mcd,OD.mcd)

#### Monitoring

bp<-seq(0,0.49,by=0.01)
sd.mat<-od.mat<-matrix(0,50,nrow(xmad))
for(i in 1:50)
{
out.mon<-PcaCov(xmad, scale=FALSE, cov.control=CovControlMcd(alpha=1-bp[i]),k=3)
sd.mat[i,]<-out.mon@sd
od.mat[i,]<-out.mon@od
}
