library(pcaPP)

###### Octane data
out<-read.csv('Octane.csv',header=T,dec=',')
X<-out[,-(1:2)]

#### PP method from Croux and filzomoser
croux.nonsparse.mad<-PCAgrid(X,k=2,method='mad')
croux.nonsparse.qn<-PCAgrid(X,k=2,method='qn')



#### sparse PP method from Croux et al.
sel1<-opt.TPO (X, k.max = 2, n.lambda = 30,method='mad')
sel1bis<-opt.TPO (X, k.max = 2, n.lambda = 30,method='qn')

###Figure 4.7
par(mfrow=c(2,2))
for (i in 1:2)  plot (sel1, k = i, f.x = "lambda")
for (i in 1:2)  objplot (sel1, k = i)

par(mfrow=c(2,2))
for (i in 1:2)  plot (sel1bis, k = i, f.x = "lambda")
for (i in 1:2)  objplot (sel1bis, k = i)

### Figures 4.8 and 4.9

plot(croux.nonsparse.mad$loadings[,1],type='l',col='gray',ylim=c(-0.4,0.4),
ylab='Loadings', xlab='Wavenumber',main='PC1')
lines(sel1$pc$load[,1])

plot(croux.nonsparse.mad$loadings[,2],type='l',col='gray',ylim=c(-0.4,0.4),
ylab='Loadings', xlab='Wavenumber',main='PC2')
lines(sel1$pc$load[,2])


plot(croux.nonsparse.qn$loadings[,1],type='l',col='gray',ylim=c(-0.4,0.4),
ylab='Loadings', xlab='Wavenumber',main='PC1')
lines(sel1bis$pc$load[,1])

plot(croux.nonsparse.qn$loadings[,2],type='l',col='gray',ylim=c(-0.4,0.4),
ylab='Loadings', xlab='Wavenumber',main='PC2')
lines(sel1bis$pc$load[,2])

#### Figure 4.10

croux.mad <- sPCAgrid(X, lambda=sel1$pc$lambda, k=2,method='mad',scores=TRUE)
croux.qn <- sPCAgrid(X, lambda=sel1$pc$lambda, k=2,method='qn',scores=TRUE)


out.mad.sp<-PCdiagplot(X,croux.mad,plot=F,raw=T,ksel=2,crit=0.975 )
plot(out.mad.sp$SDist[,1],out.mad.sp$ODist[,1], ylab='OD', xlab='SD',main='MAD- sPP')
abline(h=out.mad.sp$critOD,v=out.mad.sp$critSD)
identify(out.mad.sp$SDist[,1],out.mad.sp$ODist[,1])

out.qn.sp<-PCdiagplot(X,croux.qn,plot=F,raw=T,ksel=2,crit=0.975 )
plot(out.qn.sp$SDist[,1],out.qn.sp$ODist[,1], ylab='OD', xlab='SD',main='Qn-sPP')
abline(h=out.qn.sp$critOD,v=out.qn.sp$critSD)
identify(out.qn.sp$SDist[,1],out.qn.sp$ODist[,1])


