library(rrcov);library(elasticnet);library(robustbase)
source('selection.r')

##### Automobile data
auto<-read.table('auto.dat',sep='')
x.mad <- apply(auto[,-2], 2, mad)
xmad <- sweep(auto[,-2], 2, x.mad, "/", check.margin = FALSE)

### grid of sparsity parameters
tun<-seq(0,0.5,l=50)

### sPCA based on MM-estimation
mm<-CovMMest(xmad)
outrobmm<-eigen(mm@cov)

select.mm<-select.mm2<-NULL
for(j in 1:50)
{
	out<-spca(mm@cov, K=3, para=rep(tun[j],3),type='Gram',sparse='penalty')
    select.mm[j]<-selection.CFF(X=xmad, B=out$loadings,PC=outrobmm$vectors[,1:3],type='mad')
    select.mm2[j]<-selection.CFF(X=xmad, B=out$loadings,PC=outrobmm$vectors[,1:3],type='qn')
    }


### sPCA based on MCD
mcd<-CovMcd(xmad,alpha=.75)
outrobmcd<-eigen(mcd@cov)
select.mcd<-select.mcd2<-NULL

for(j in 1:50)
{
	out<-spca(mcd@cov, K=3, para=rep(tun[j],3),type='Gram',sparse='penalty')
    select.mcd[j]<-selection.CFF(X=xmad, B=out$loadings,outrobmcd$vectors[,1:3],type='mad')
    select.mcd2[j]<-selection.CFF(X=xmad, B=out$loadings,outrobmcd$vectors[,1:3],type='qn')
    }

#### Figure 4.3 
plot(tun, select.mcd-min(select.mcd),type='l',xlab=expression(lambda), ylab='',main='RMCD')
lines(tun, select.mcd2-min(select.mcd2),lty=2) 
legend('topleft',c('MAD','Qn'),lty=1:2)
plot(tun, select.mm-min(select.mm),type='l',xlab=expression(lambda), ylab='',main='MM') 
lines(tun, select.mm2-min(select.mm2),lty=2) 
legend('topleft',c('MAD','Qn'),lty=1:2)

### sRPCA output 
out.mm<-spca(mm@cov, K=3, para=rep(tun[which.min(select.mm2)],3),
type='Gram',sparse='penalty')

out.mcd<-spca(mcd@cov, K=3, para=rep(tun[which.min(select.mcd2)],3),
type='Gram',sparse='penalty')

#### Alternative solution (not included in the book)
out.mm2<-spca(mm@cov, K=3, para=rep(tun[which.min(select.mm)],3),type='Gram',sparse='penalty')
out.mcd2<-spca(mcd@cov, K=3, para=rep(tun[which.min(select.mcd)],3),type='Gram',sparse='penalty')


### Loadings and pev in Table 4.1
round(out.mm$loadings,2)
round(out.mcd$loadings,2)


### from sparse MM

### scores
X0.mm<-sapply(1:ncol(xmad), function(j) xmad[,j]-mm@center[j])
score.mm<-X0.mm%*%outrobmm$vectors
score.mm.sparse<-X0.mm%*%out.mm$loadings

### sort the scores according to their robust variance

apply(score.mm.sparse,2,mad)
sapply(1:3,function(j) lmrob(score.mm.sparse[,j]~1)$scale^2)

round(apply(score.mm[,1:3],2,mad)^2/ncol(xmad),4)*100
round(apply(score.mm.sparse,2,mad)^2/ncol(xmad),4)*100



#### We want to decorrelate by using LTS.  LTS coincides with univariate MCD
#### First obtain the total robust variance 
LTSest<-sapply(1:ncol(xmad),function(j) ltsReg(xmad[,j]~1,alpha=0.75)$scale^2)
totvar<-sum(LTSest)
#### unconstrained EV
LTSest.score<-sapply(1:ncol(xmad),function(j) ltsReg(score.mm[,j]~1,alpha=0.75)$scale^2)
round(LTSest.score[1:3]/totvar,4)*100
#### unadjusted EV
LTSest.score.sparse<-sapply(1:3,function(j) ltsReg(score.mm.sparse[,j]~1,alpha=0.75)$scale^2)
round(LTSest.score.sparse[1:3]/totvar,4)*100
#### adjusted EV
mm0<-ltsReg(score.mm.sparse[,1]~1,alpha=0.75)
mm1<-ltsReg(score.mm.sparse[,3]~score.mm.sparse[,1],alpha=0.75)
mm2<-ltsReg(score.mm.sparse[,2]~score.mm.sparse[,-2],alpha=0.75)
round(c(mm0$s,mm1$s,mm2$s)^2/totvar,4)*100

#### A different solution

round(c(mad(mm0$residuals),mad(mm1$residuals),mad(mm2$residuals))^2/14,4)*100


#### Another different solution
totvar.scores<-sum(LTSest.score)
round(c(mad(mmo$residuals),mad(mm1$residuals),mad(mm2$residuals))^2/totvar.scores,4)*100

### Figure 4.4
pairs(score.mm.sparse)
pairs(score.mm.sparse.decor)


### from sparse MCD

### scores
X0.mcd<-sapply(1:ncol(xmad), function(j) xmad[,j]-mcd@center[j])
score.mcd<-X0.mcd%*%outrobmcd$vectors
score.mcd.sparse<-X0.mcd%*%out.mcd$loadings
### sort the scores according to their robust variance
### different proposals
### Explained Variance based on MAD 
rob.var<-apply(score.mcd.sparse,2,mad)^2
ord<-rank(-rob.var)
### Explained Variance based on LTS 
rob.var<-sapply(1:3,function(j) ltsReg(score.mcd.sparse[,j]~1,alpha=0.75)$scale^2)



#### We want to decorrelate by using LTS. Univariate LTS coincides with univariate MCD
#### First obtain the total robust variance 
LTSest<-sapply(1:ncol(xmad),function(j) ltsReg(xmad[,j]~1,alpha=0.75)$scale^2)
totvar<-sum(LTSest)
#### unconstrained EV
LTSest.score<-sapply(1:ncol(xmad),function(j) ltsReg(score.mcd[,j]~1,alpha=0.75)$scale^2)
round(LTSest.score[1:3]/totvar,4)*100
#### unadjusted EV
LTSest.score.sparse<-sapply(1:3,function(j) ltsReg(score.mcd.sparse[,j]~1,alpha=0.75)$scale^2)
round(LTSest.score.sparse[1:3]/totvar,4)*100
#### adjusted EV
mcd0<-ltsReg(score.mcd.sparse[,1]~1,alpha=0.75)
mcd1<-ltsReg(score.mcd.sparse[,3]~score.mcd.sparse[,1],alpha=0.75)
mcd2<-ltsReg(score.mcd.sparse[,2]~score.mcd.sparse[,-2],alpha=0.75)
round(c(mcd0$s,mcd1$s,mcd2$s)^2/totvar,4)*100

#### A different solution

round(c(mad(mcd0$residuals),mad(mcd1$residuals),mad(mcd2$residuals))^2/14,4)*100


#### Another different solution
totvar.scores<-sum(LTSest.score)
round(c(mad(mcd0$residuals),mad(mcd1$residuals),mad(mcd2$residuals))^2/totvar.scores,4)*100




######## Trade off curve in Figure 4.5
 
LTSest<-sapply(1:ncol(xmad),function(j) ltsReg(xmad[,j]~1,alpha=0.75)$scale^2)
totvar<-sum(LTSest)
#### MM estimation


X0.mm<-sapply(1:ncol(xmad), function(j) xmad[,j]-mm@center[j])
tun<-seq(0,0.5,l=50)
ev<-NULL
for(i in 1:50){
out.ev<-spca(mm@cov, K=3, para=rep(tun[i],3),type='Gram',sparse='penalty')
score.sparse<-X0.mm%*%out.ev$loadings
### sort
LTSest<-sapply(1:3,function(j) ltsReg(score.sparse[,j]~1,alpha=0.75)$scale^2)
ord<-rank(-LTSest)
r1<-ltsReg(score.sparse[,which(ord==2)]~score.sparse[,1],alpha=0.75)
r2<-ltsReg(score.sparse[,which(ord==3)]~score.sparse[,-which(ord==3)],alpha=0.75)
ev[i]<-sum(c(LTSest[1],r1$s^2,r2$s^2))
}


plot(tun,100*ev/totvar,type='b',xlab=expression(lambda),ylab='Explained Variance',main='MM')
abline(v=0.102,lty=2)

#### MCD estimation

X0.mcd<-sapply(1:ncol(xmad), function(j) xmad[,j]-mcd@center[j])
tun<-seq(0,0.5,l=50)
ev2<-NULL
for(i in 1:50){
out.ev<-spca(mcd@cov, K=3, para=rep(tun[i],3),type='Gram',sparse='penalty')
score.sparse<-X0.mcd%*%out.ev$loadings
### sort
LTSest<-sapply(1:3,function(j) ltsReg(score.sparse[,j]~1,alpha=0.75)$scale^2)
ord<-rank(-LTSest)
r1<-ltsReg(score.sparse[,which(ord==2)]~score.sparse[,1],alpha=0.75)
r2<-ltsReg(score.sparse[,which(ord==3)]~score.sparse[,-which(ord==3)],alpha=0.75)
ev2[i]<-sum(c(LTSest[1],r1$s^2,r2$s^2))
}


plot(tun,100*ev2/totvar,type='b',xlab=expression(lambda),ylab='Explained Variance',main='RMCD')
abline(v=0.082,lty=2)




##### Outlier map from MCD in Figure 4.6 

Z<-score.mcd.sparse
autoval<-sapply(1:3,function(j) ltsReg(Z[,j]~1,alpha=0.75)$scale^2)
SDsparse.mcd<-sqrt(mahalanobis(Z,rep(0,3),diag(autoval)))
tildeX<-Z%*%t(cbind(out.mcd$load))
resX<-X0.mcd-tildeX
ODsparse.mcd<-sapply(1:nrow(xmad), function(i) sqrt(sum(resX[i,]^2)))
ODsparsen.mcd<-ODsparse.mcd^(2/3)
cutoff.OD=(median(ODsparsen.mcd)+qnorm(.975)*mad(ODsparsen.mcd))^(3/2)
plot(SDsparse.mcd,ODsparse.mcd,main='RMCD',ylab='OD', xlab='SD',type='n',
 xlim=range(SDsparse.mcd),ylim=range(ODsparse.mcd))
points(SDsparse.mcd[auto[,2]=='gas'],ODsparse.mcd[auto[,2]=='gas'])
points(SDsparse.mcd[auto[,2]=='diesel'],ODsparse.mcd[auto[,2]=='diesel'], pch='+')
abline(v=sqrt(qchisq(.975,3)), h=cutoff.OD)
identify(SDsparse.mcd,ODsparse.mcd)

##### Outlier map from MM (not in the book) 


Z<-score.mm.sparse
autoval<-sapply(1:3,function(j) ltsReg(Z[,j]~1,alpha=0.75)$scale^2)
SDsparse.mm<-sqrt(mahalanobis(Z,rep(0,3),diag(autoval)))
tildeX<-Z%*%t(cbind(out.mm$load))
resX<-X0.mcd-tildeX
ODsparse.mm<-sapply(1:nrow(xmad), function(i) sqrt(sum(resX[i,]^2)))
ODsparsen.mm<-ODsparse.mm^(2/3)
cutoff.OD=(median(ODsparsen.mm)+qnorm(.975)*mad(ODsparsen.mm))^(3/2)
plot(SDsparse.mcd,ODsparse.mm,main='MM',ylab='OD', xlab='SD',type='n',
 xlim=range(SDsparse.mm),ylim=range(ODsparse.mm))
points(SDsparse.mcd[auto[,2]=='gas'],ODsparse.mcd[auto[,2]=='gas'])
points(SDsparse.mcd[auto[,2]=='diesel'],ODsparse.mcd[auto[,2]=='diesel'], pch='+')
abline(v=sqrt(qchisq(.975,3)), h=cutoff.OD)
identify(SDsparse.mcd,ODsparse.mcd)


