##### Butterfly data

library(ade4);library(rrcov);library(MASS)
source('CovMcdF.r')
data(butterfly)


### data
X<-butterfly$envir
Y<-butterfly$genet
Y<-cbind(Y[,1]+Y[,2], Y[,3:5])
colnames(Y)<-c('0.4-0.6','0.8','1','1.16')
XY<-cbind(X,Y)

#### Estimates of the correlation matrix

## Classical
clas.cor<-cor(XY)
## RMCD
S.rmcd<-CovMcdF(XY,alpha=0.75)
rmcd.cor<-cov2cor(S.rmcd$cov)
## MM
S.mm<-CovMMest(XY)
mm.cor<-cov2cor(S.mm@cov)

#### Plots (Figures 5.3 amd 5.4)

plot(S.mm, which='dd')
plot(S.mm,which='pairs',cex=1.4)


## Standard CCA

Rxx<-clas.cor[1:4,1:4]
Rxy<-clas.cor[1:4,5:8]
Ryy<-clas.cor[5:8,5:8]

Px<-solve(Rxx)%*%Rxy%*%solve(Ryy)%*%t(Rxy)
Py<-solve(Ryy)%*%t(Rxy)%*%solve(Rxx)%*%Rxy

forA<-eigen(Px,symmetric=F)
forB<-eigen(Py,symmetric=F)


## canonical correlation
rho<-sqrt(forA$values)
## canonical coefficients
A<-forA$vector
B<-forB$vector


### scores

xx<-scale(X)
yy<-scale(Y)
xc<-xx%*%A
yc<-yy%*%B


###### Robust CCA from MM

Rxx<-mm.cor[1:4,1:4]
Rxy<-mm.cor[1:4,5:8]
Ryy<-mm.cor[5:8,5:8]

Px<-solve(Rxx)%*%Rxy%*%solve(Ryy)%*%t(Rxy)
Py<-solve(Ryy)%*%t(Rxy)%*%solve(Rxx)%*%Rxy

forA<-eigen(Px,symmetric=F)
forB<-eigen(Py,symmetric=F)

rhomm<-sqrt(forA$values)

Amm<-forA$vector
Bmm<-forB$vector

### scores
xxmm<-sapply(1:4, function(j) (X[,j]-S.mm@center[j])/sqrt(S.mm@cov[j,j]))
yymm<-sapply(1:4, function(j) (Y[,j]-S.mm@center[(j+4)])/sqrt(S.mm@cov[(j+4),(j+4)]))
xcmm<-xxmm%*%Amm
ycmm<-yymm%*%Bmm



#### Loadings and cross loadings by Spearman correlation index

cor(xcmm[,1], X,method='spearman')
cor(xcmm[,1], Y,method='spearman')
cor(ycmm[,1], X,method='spearman')
cor(ycmm[,1], Y,method='spearman')

cor(xcmm[,1], xxmm,method='spearman')
cor(xcmm[,1], yymm,method='spearman')
cor(ycmm[,1], yymm,method='spearman')
cor(ycmm[,1], xxmm,method='spearman')

### loadings by using Huber psi
lx<-NULL
for(i in 1:4){
num<-mean(xxmm[,i]*psi.huber(xxmm[,i])*xcmm[,1]*psi.huber(xcmm[,1]))
den<-sqrt(mean((xxmm[,i]*psi.huber(xxmm[,i]))^2)*mean((xcmm[,1]*psi.huber(xcmm[,1]))^2))
lx[i]<-num/den}
ly<-NULL
for(i in 1:4){
num<-mean(yymm[,i]*psi.huber(yymm[,i])*ycmm[,1]*psi.huber(ycmm[,1]))
den<-sqrt(mean((yymm[,i]*psi.huber(yymm[,i]))^2)*mean((ycmm[,1]*psi.huber(ycmm[,1]))^2))
ly[i]<-num/den}

### cross loadings
cx<-NULL
for(i in 1:4){
num<-mean(xxmm[,i]*psi.huber(xxmm[,i])*ycmm[,1]*psi.huber(ycmm[,1]))
den<-sqrt(mean((xxmm[,i]*psi.huber(xxmm[,i]))^2)*mean((ycmm[,1]*psi.huber(ycmm[,1]))^2))
cx[i]<-num/den}
cy<-NULL
for(i in 1:4){
num<-mean(yymm[,i]*psi.huber(yymm[,i])*xcmm[,1]*psi.huber(xcmm[,1]))
den<-sqrt(mean((yymm[,i]*psi.huber(yymm[,i]))^2)*mean((xcmm[,1]*psi.huber(xcmm[,1]))^2))
cy[i]<-num/den}



