##### Linnerud data
library(rrcov)
library(MASS)
library(ccaPP)
library(mixOmics)
data(linnerud)

source('CovMcdF.r')
###### Classical CCA
#### Sample correlation matrix
x<-linnerud$ex
y<-linnerud$ph
linn.cor<-cor(cbind(x,y))

#### Blocks
Rxx<-linn.cor[1:3,1:3]
Rxy<-linn.cor[1:3,4:6]
Ryy<-linn.cor[4:6,4:6]

## Product matrices
Px<-solve(Rxx)%*%Rxy%*%solve(Ryy)%*%t(Rxy)
Py<-solve(Ryy)%*%t(Rxy)%*%solve(Rxx)%*%Rxy

### eigen-decompsition
forA<-eigen(Px,symmetric=F)
forB<-eigen(Py,symmetric=F)


## canonical correlations
rho<-sqrt(forA$values)
## canonical coefficients
A<-forA$vector
B<-forB$vector

### scores

xx<-scale(x)
yy<-scale(y)
xc<-xx%*%A
yc<-yy%*%B

## loadings on the first pair of canonical variables

cor(xc[,1], x,use='pairwise')
cor(yc[,1], y,use='pairwise')

## cross loadings on the first pair of canonical variables

cor(yc[,1], x,use='pairwise')
cor(xc[,1], y,use='pairwise')

##################

## RMCD

xxrmcd<-sapply(1:3,function(j) x[,j]/mad(x[,j]))
yyrmcd<-sapply(1:3,function(j) y[,j]/mad(y[,j]))
Srmcd<-CovMcdF(cbind(xxrmcd,yyrmcd),alpha=0.75)

Rrmcd<-Srmcd$cov
## Blocks
Rxxrmcd<-Rrmcd[1:3,1:3]
Rxyrmcd<-Rrmcd[1:3,4:6]
Ryyrmcd<-Rrmcd[4:6,4:6]
### Product matrices
Pxrmcd<-solve(Rxxrmcd)%*%Rxyrmcd%*%solve(Ryyrmcd)%*%t(Rxyrmcd)
Pyrmcd<-solve(Ryyrmcd)%*%t(Rxyrmcd)%*%solve(Rxxrmcd)%*%Rxyrmcd
## eigen analysis
forArmcd<-eigen(Pxrmcd)
forBrmcd<-eigen(Pyrmcd)

## canonical correlation
rhormcd<-sqrt(forArmcd$values)
## canonical coefficients
Armcd<-forArmcd$vector
Brmcd<-forBrmcd$vector

### scores
xxcenter<-sapply(1:3,function(j) (xxrmcd[,j]-Srmcd$center[j])/sqrt(Srmcd$cov[j,j]))
xcrmcd<-xxcenter%*%Armcd
yycenter<-sapply(1:3,function(j) (yyrmcd[,j]-Srmcd$center[(j+3)])/sqrt(Srmcd$cov[(j+3),(j+3)]))
ycrmcd<-yycenter%*%Brmcd

#### loadings by pairwise RMCD
outx1<-CovMcdF(cbind(xcrmcd[,1],xxrmcd[,1]))
outx2<-CovMcdF(cbind(xcrmcd[,1],xxrmcd[,2]))
outx3<-CovMcdF(cbind(xcrmcd[,1],xxrmcd[,3]))

outy1<-CovMcdF(cbind(ycrmcd[,1],yyrmcd[,1]))
outy2<-CovMcdF(cbind(ycrmcd[,1],yyrmcd[,2]))
outy3<-CovMcdF(cbind(ycrmcd[,1],yyrmcd[,3]))


### cross loadings by pairwise RMCD
outxc1<-CovMcdF(cbind(ycrmcd[,1],xxrmcd[,1]))
outxc2<-CovMcdF(cbind(ycrmcd[,1],xxrmcd[,2]))
outxc3<-CovMcdF(cbind(ycrmcd[,1],xxrmcd[,3]))

outyc1<-CovMcdF(cbind(xcrmcd[,1],yyrmcd[,1]))
outyc2<-CovMcdF(cbind(xcrmcd[,1],yyrmcd[,2]))
outyc3<-CovMcdF(cbind(xcrmcd[,1],yyrmcd[,3]))


### loadings by Huber psi
lx<-NULL
for(i in 1:3)
{
num<-mean(xxcenter[,i]*psi.huber(xxcenter[,i])*xcrmcd[,1]*psi.huber(xcrmcd[,1]))
den<-sqrt(mean((xxcenter[,i]*psi.huber(xxcenter[,i]))^2)*mean((xcrmcd[,1]*psi.huber(xcrmcd[,1]))^2))
lx[i]<-num/den
}

ly<-NULL
for(i in 1:3)
{
num<-mean(yycenter[,i]*psi.huber(yycenter[,i])*ycrmcd[,1]*psi.huber(ycrmcd[,1]))
den<-sqrt(mean((yycenter[,i]*psi.huber(yycenter[,i]))^2)*mean((ycrmcd[,1]*psi.huber(ycrmcd[,1]))^2))
ly[i]<-num/den
}

### cross loadings by Huber psi
cx<-NULL
for(i in 1:3)
{
num<-mean(xxcenter[,i]*psi.huber(xxcenter[,i])*ycrmcd[,1]*psi.huber(ycrmcd[,1]))
den<-sqrt(mean((xxcenter[,i]*psi.huber(xxcenter[,i]))^2)*mean((ycrmcd[,1]*psi.huber(ycrmcd[,1]))^2))
cx[i]<-num/den
}

cy<-NULL
for(i in 1:3)
{
num<-mean(yycenter[,i]*psi.huber(yycenter[,i])*xcrmcd[,1]*psi.huber(xcrmcd[,1]))
den<-sqrt(mean((yycenter[,i]*psi.huber(yycenter[,i]))^2)*mean((xcrmcd[,1]*psi.huber(xcrmcd[,1]))^2))
cy[i]<-num/den
}


## Figure 5.2


plot(xcrmcd[,1],xxrmcd[,1], ylim=c(4,12),xlim=c(-2,3),ylab='weight', xlab=expression(X[11]^'*'))
p=2
n<-sum(outx1$wt)
q<-((n-1)^2)*qbeta(.975,p/2,(n-p-1)/2)/n
lines(ellipse(x=outx1$cov, centre=outx1$center,t=sqrt(q)))
identify(xcrmcd[,1],xxrmcd[,1])


plot(xcrmcd[,1],xxrmcd[,2], ylim=c(9, 16), xlim=c(-2,3),ylab='waist', xlab=expression(X[11]^'*'))
p=2
n<-sum(outx2$wt)
q<-((n-1)^2)*qbeta(.975,p/2,(n-p-1)/2)/n
lines(ellipse(x=outx2$cov, centre=outx2$center,t=sqrt(q)))
identify(xcrmcd[,1],xxrmcd[,2])


plot(xcrmcd[,1],xxrmcd[,3],xlim=c(-2,3),ylim=c(5,11), ylab='pulse', xlab=expression(X[11]^'*'))
p=2
n<-sum(outx3$wt)
q<-((n-1)^2)*qbeta(.975,p/2,(n-p-1)/2)/n
lines(ellipse(x=outx3$cov, centre=outx3$center,t=sqrt(q)))
identify(xcrmcd[,1],xxrmcd[,3])


plot(xcrmcd[,1],yyrmcd[,1],xlim=c(-2,3),ylim=c(-1,4), ylab='tractions', xlab=expression(X[11]^'*'))
p=2
n<-sum(outyc1$wt)
q<-((n-1)^2)*qbeta(.975,p/2,(n-p-1)/2)/n
lines(ellipse(x=outyc1$cov, centre=outyc1$center,t=sqrt(q)))
identify(xcrmcd[,1],yyrmcd[,1])

plot(xcrmcd[,1],yyrmcd[,2],xlim=c(-2,3),ylim=c(-1,6), ylab='tractions', xlab=expression(X[11]^'*'))
p=2
n<-sum(outyc2$wt)
q<-((n-1)^2)*qbeta(.975,p/2,(n-p-1)/2)/n
lines(ellipse(x=outyc2$cov, centre=outyc2$center,t=sqrt(q)))
identify(xcrmcd[,1],yyrmcd[,2])

plot(xcrmcd[,1],yyrmcd[,3],xlim=c(-2,3),ylim=c(-3,14), ylab='jumps', xlab=expression(X[11]^'*'))
p=2
n<-sum(outyc3$wt)
q<-((n-1)^2)*qbeta(.975,p/2,(n-p-1)/2)/n
lines(ellipse(x=outyc3$cov, centre=outyc3$center,t=sqrt(q)))
identify(xcrmcd[,1],yyrmcd[,3])

plot(ycrmcd[,1],xxrmcd[,1], ylim=c(4,12),xlim=c(-2,3),ylab='weight', xlab=expression(X[21]^'*'))
p=2
n<-sum(outxc1$wt)
q<-((n-1)^2)*qbeta(.975,p/2,(n-p-1)/2)/n
lines(ellipse(x=outxc1$cov, centre=outxc1$center,t=sqrt(q)))
identify(ycrmcd[,1],xxrmcd[,1])


plot(ycrmcd[,1],xxrmcd[,2], ylim=c(9, 16), xlim=c(-2,3),ylab='waist', xlab=expression(X[21]^'*'))
p=2
n<-sum(outxc2$wt)
q<-((n-1)^2)*qbeta(.975,p/2,(n-p-1)/2)/n
lines(ellipse(x=outxc2$cov, centre=outxc2$center,t=sqrt(q)))
identify(ycrmcd[,1],xxrmcd[,2])


plot(ycrmcd[,1],xxrmcd[,3],xlim=c(-2,3),ylim=c(5,11), ylab='pulse', xlab=expression(X[21]^'*'))
p=2
n<-sum(outxc3$wt)
q<-((n-1)^2)*qbeta(.975,p/2,(n-p-1)/2)/n
lines(ellipse(x=outxc3$cov, centre=outxc3$center,t=sqrt(q)))
identify(ycrmcd[,1],xxrmcd[,3])


plot(ycrmcd[,1],yyrmcd[,1],xlim=c(-2,3),ylim=c(-1,4), ylab='tractions', xlab=expression(X[21]^'*'))
p=2
n<-sum(outy1$wt)
q<-((n-1)^2)*qbeta(.975,p/2,(n-p-1)/2)/n
lines(ellipse(x=outy1$cov, centre=outy1$center,t=sqrt(q)))
identify(ycrmcd[,1],yyrmcd[,1])

plot(ycrmcd[,1],yyrmcd[,2],xlim=c(-2,3),ylim=c(-1,6), ylab='tractions', xlab=expression(X[21]^'*'))
p=2
n<-sum(outy2$wt)
q<-((n-1)^2)*qbeta(.975,p/2,(n-p-1)/2)/n
lines(ellipse(x=outy2$cov, centre=outy2$center,t=sqrt(q)))
identify(ycrmcd[,1],yyrmcd[,2])

plot(ycrmcd[,1],yyrmcd[,3],xlim=c(-2,3),ylim=c(-3,14), ylab='jumps', xlab=expression(X[21]^'*'))
p=2
n<-sum(outy3$wt)
q<-((n-1)^2)*qbeta(.975,p/2,(n-p-1)/2)/n
lines(ellipse(x=outy3$cov, centre=outy3$center,t=sqrt(q)))
identify(ycrmcd[,1],yyrmcd[,3])

### spherical CCA

linn.loc<-PcaLocantore(cbind(xxrmcd,yyrmcd))

### Covariance matrix
S.loc<- linn.loc@loadings%*%diag(linn.loc@eigenvalues)%*%t(linn.loc@loadings)

### blocks
Rxxloc<-S.loc[1:3,1:3]
Rxyloc<-S.loc[1:3,4:6]
Ryyloc<-S.loc[4:6,4:6]
### Product matrices
Pxloc<-solve(Rxxloc)%*%Rxyloc%*%solve(Ryyloc)%*%t(Rxyloc)
Pyloc<-solve(Ryyloc)%*%t(Rxyloc)%*%solve(Rxxloc)%*%Rxyloc
### eigen decomposition
forAloc<-eigen(Pxloc)
forBloc<-eigen(Pyloc)

## canonical correlations
rholoc<-sqrt(forAloc$values)

### canonical vectors
Aloc<-forAloc$vector
Bloc<-forBloc$vector

### scores
xxcenter<-sapply(1:3,function(j) (xxrmcd[,j]-linn.loc@center[j])/sqrt(S.loc[j,j]))
xcloc<-xxcenter%*%Aloc
yycenter<-sapply(1:3,function(j) (yyrmcd[,j]-linn.loc@center[j+3])/sqrt(S.loc[j+3,j+3]))
ycloc<-yyrmcd%*%Bloc

## loadings

round(rxloc<-cor(xcloc,xxrmcd),3)[1,]
round(ryloc<-cor(ycloc,yyrmcd),3)[1,]

## cross loadings

round(rcxloc<-cor(xcloc,yyrmcd),3)[1,]
round(rcyloc<-cor(ycloc,xxrmcd),3)[1,]


#### Figure 5.6
plot(xcloc[,1],ycloc[,1],main='SCCA', xlab=expression(X[11]^'*'), ylab=expression(X[21]^'*'),
xlim=c(-3,3),ylim=c(-4,2))
p=2
n<-20
q<-((n-1)^2)*qbeta(.975,p/2,(n-p-1)/2)/n
lines(ellipse(x=cov(cbind(xcloc[,1],ycloc[,1])), centre=c(mean(xcloc[,1]),mean(ycloc[,1])),t=sqrt(q)))
identify(xcloc[,1],ycloc[,1])

####### PP CCA

r1<-ccaProj(xxrmcd,yyrmcd, k = 1, method = "spearman")    
r2<-ccaProj(xxrmcd,yyrmcd, k = 1, method = "M")

