
library(rrcov)
library(psych)

source('CovMcdF.r')

#### Automobile data
auto<-read.table('auto.dat',sep='',header=TRUE)

X<-auto[,-2]
auto.mcd<-CovMcdF(X, alpha=0.75)
inx.bad<-which(auto.mcd$wt==0)
inx.good<-which(auto.mcd$wt!=0)

###maximum likelihood
auto.mle<-list(m=colMeans(X),S=cov(X))
auto.mle.dist<-mahalanobis(X,auto.mle$m,auto.mle$S)
cc=((nrow(X)-1)^2)/nrow(X)
quantmle<-qbeta(.975,ncol(X)/2, (nrow(X)-ncol(X)-1)/2)*cc


##RMCD non outlyng data
nw<-sum(auto.mcd$wt)
ccw1=((nw-1)^2)/nw
quantrmcd<-qbeta(.975,ncol(X)/2, (nw-ncol(X)-1)/2)*ccw1
## RMCD outlying data
ccw2=(nw^2-1)*p/nw/(nw-p)
quantout<-qf(.975,ncol(X), nw-ncol(X))*ccw2

###
inx.diesel<-which(autonew[,2]=='diesel')
inx.gas<-which(autonew[,2]=='gas')

#### identify diesel cars in the trimmed set
inx<-NULL
for(i in 1: length(inx.diesel))
{
	inx[i]<-which(inx.bad==inx.diesel[i])
}


#### Distance-distance plot in Figure 6.1
plot(sqrt(auto.mle.dist), sqrt(auto.mcd$mah), xlab='Mahalanobis distance', ylab='Robust distance',type='n')
points(sqrt(auto.mle.dist[inx.good]),sqrt(auto.mcd$mah[inx.good]),pch=1)
points(sqrt(auto.mle.dist[inx.bad[-inx2]]),sqrt(auto.mcd$mah[inx.bad[-inx2]]),pch=2)
points(sqrt(auto.mle.dist[inx.bad[inx2]]),sqrt(auto.mcd$mah[inx.bad[inx2]]),pch='+')
abline(v=sqrt(quantmle), h=sqrt(quantout))
abline(h=sqrt(quantrmcd),lty=2)

##### Classical FA

clas.af<-fa(r=cor(X), nfactors=3,rotate='varimax',covar=FALSE,scores='regression',fm='pa')
hat.f<-factor.scores(X,clas.af,method='Thurstone')
biplot(x=hat.f$scores[,1:2], y=clas.af$loadings[,1:2],xlab='Factor 1', ylab='Factor 2', main='Classical',xlim=c(-4,4),col=c('black','gray'))
biplot(x=hat.f$scores[,1:3], y=clas.af$loadings[,1:3],xlab='Factor 1', ylab='Factor 3', main='Classical',xlim=c(-4,4),col=c('black','gray'))
 biplot(x=hat.f$scores[,2:3], y=clas.af$loadings[,2:3],xlab='Factor 2', ylab='Factor 3', main='Classical',xlim=c(-4,4),col=c('black','gray'))
##### Robust FA
R<-cov2cor(auto.mcd$cov)
rob.af<-fa(r=R, nfactors=3,rotate='varimax',covar=FALSE,scores='regression',fm='pa')
Xc<-sapply(1:ncol(X), function(j) (X[,j]-auto.mcd$center[j])/sqrt(auto.mcd$cov[j,j]))
hat.f.rob<-t(rob.af$loadings)%*%solve(R)%*%t(Xc)
biplot(x=t(hat.f.rob)[,1:2], y=rob.af$loadings[,1:2],xlab='Factor 1', ylab='Factor 2', main='RMCD',xlim=c(-4,4),col=c('black','gray'))
biplot(x=t(hat.f.rob)[,1:3], y=rob.af$loadings[,1:3],xlab='Factor 1', ylab='Factor 3', main='RMCD',xlim=c(-4,4),col=c('black','gray'))
biplot(x=t(hat.f.rob)[,2:3], y=rob.af$loadings[,2:3],xlab='Factor 2', ylab='Factor 2', main='RMCD',xlim=c(-4,4),col=c('black','gray'))
